var _       = require( 'underscore' )._;
var fs      = require( 'fs' );
var http    = require( 'http' );
var vows    = require( 'vows' );
var assert  = require( 'assert' );
var Klient  = require( '../lib/klient' ).Klient;

vows
  .describe( 'KLIENT' )
  .addBatch({
    'init without server': {
      topic: function () {
        return new( Klient );
      },
      'and remote server is listening on configured host:port': {
        topic: function ( client ) {
          var self = this;
          var srv = createServer();

          srv.listen(client.defaults.port, client.defaults.host, function () {
            self.callback( null, srv );
          });
        },
        'when api method called': {
          topic: function ( srv, client ) {
            return client.get( { path: '/' } );
          },
          'should request against *remote server*': function ( err, res ) {
            assert.ifError( err );
            assert.equal( res.body.toString().split( '\n' )[0], 'GET / HTTP/1.1' );
          }
        },
        teardown: function ( srv, client ) {
          srv.close();
        }
      }
    }
  })
  .addBatch({
    'init with server': {
      topic: function () {
        return new( Klient )( { server: createServer() } );
      },
      'when get is called': {
        topic: function ( client ) {
          return client.get( { path: '/' } );
        },
        'should request with method *GET*': function ( err, res ) {
          assert.ifError( err );
          assert.equal( res.body.toString().split( '\n' )[0], 'GET / HTTP/1.1' );
        },
        'should return body as *Buffer*': function ( err, res ) {
          assert.ifError( err );
          assert.instanceOf( res.body, Buffer );
        }
      },
      'when post is called': {
        topic: function ( client ) {
          return client.post( { path: '/' } );
        },
        'should request with method *POST*': function ( err, res ) {
          assert.ifError( err );
          assert.equal( res.body.toString().split( '\n' )[0], 'POST / HTTP/1.1' );
        }
      },
      'when put is called': {
        topic: function ( client ) {
          return client.put( { path: '/' } );
        },
        'should request with method *PUT*': function ( err, res ) {
          assert.ifError( err );
          assert.equal( res.body.toString().split( '\n' )[0], 'PUT / HTTP/1.1' );
        }
      },
      'when del is called': {
        topic: function ( client ) {
          return client.del( { path: '/' } );
        },
        'should request with method *DELETE*': function ( err, res ) {
          assert.ifError( err );
          assert.equal( res.body.toString().split( '\n' )[0], 'DELETE / HTTP/1.1' );
        }
      },
      teardown: function ( client ) {
        client.close()
      }
    }
  })
  .addBatch({
    'init with server': {
      topic: function () {
        return new( Klient )( { server: createServer() } );
      },
      'when POST is called with data array': {
        topic: function ( client ) {
          return client.post({
            path: '/upload',
            data: [
              'foobar\n',
              'barbaz'
            ]
          });
        },
        'should send *multiline body*': function ( err, res ) {
          assert.ifError( err );
          assert.equal( res.body.toString().split( '\n\n' )[1], 'foobar\nbarbaz' );
        }
      },
      teardown: function ( client ) {
        client.close();
      }
    }
  })
  .addBatch({
    'init with server': {
      topic: function () {
        return new( Klient )( { server: createServer() } );
      },
      'when mulitpart POST is called': {
        topic: function ( client ) {
          var opts = {
            headers: {
              'content-type': 'multipart/form-data; boundary=AaB03x'
            },
            data: []
          };

          opts.data.push( '--AaB03x\n' );
          opts.data.push( 'Content-Disposition: form-data; name="submit-name"\n\n' );
          opts.data.push( 'Larry\n' );
          opts.data.push( '--AaB03x\n' );
          opts.data.push( 'Content-Disposition: form-data; name="files"; filename="file.txt"\n' );
          opts.data.push( 'Content-Type: text/plain\n\n' );
          opts.data.push( fs.readFileSync( './spec/fixtures/file.txt' ) );
          opts.data.push( '--AaB03x\n' );

          return client.post( opts );
        },
        'should send *multipart*': function ( err, res ) {
          assert.ifError( err );
          assert.equal( res.body.toString(), fs.readFileSync( './spec/fixtures/multipart.http' ).toString() );
        }
      },
      teardown: function ( client ) {
        client.close();
      }
    }
  })
  .export( module );

// fixtures & helpers
function createServer () {
  var server = new( http.Server );

  server.on( 'request', function ( req, res ) {
    var body = req.method + ' ' + req.url + ' HTTP/' + req.httpVersion + '\n';

    _.each( req.headers, function ( value, key ) {
      body += key + ': ' + value + '\n';
    });

    body += '\n';

    req.on( 'data', function ( data ) {
      body += data;
    });

    req.on( 'end', function () {
      res.writeHead( 200, {
        'content-length': body.length,
        'content-type': 'text/plain'
      });

      res.end( body );
    })
  })

  return server;
}

